<?php

declare(strict_types=1);

namespace Drupal\devel_visual\ConfigObject;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\ConfigTarget;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;

class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'devel_visual_settings_form';
  }

  /**
   * {@inheritdoc}
   *
   * @phpstan-return array<string>
   */
  protected function getEditableConfigNames(): array {
    return [
      'devel_visual.settings',
    ];
  }

  /**
   * {@inheritdoc}
   *
   * @phpstan-param array<string, mixed> $form
   *
   * @phpstan-return array<string, mixed>
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $configName = $this->getEditableConfigNames()[0];

    $form['cropType'] = [
      '#type' => 'details',
      '#tree' => TRUE,
      '#title' => $this->t('Crop Type'),
      '#open' => TRUE,

      'class' => [
        '#type' => 'details',
        '#tree' => TRUE,
        '#title' => $this->t('Class'),
        '#open' => TRUE,

        'handler' => [
          '#config_target' => new ConfigTarget($configName, 'cropType.class.handler'),
          '#type' => 'radios',
          '#required' => FALSE,
          '#title' => $this->t('Handler'),
          '#options' => [
            '' => $this->t('- Original -'),
            'devel_visual' => $this->t('Devel Visual'),
          ],
        ],
      ],
    ];

    $form['imageStyle'] = [
      '#type' => 'details',
      '#tree' => TRUE,
      '#title' => $this->t('Image Style'),
      '#open' => TRUE,

      'listBuilder' => [
        '#type' => 'details',
        '#tree' => TRUE,
        '#title' => $this->t('List Builder'),
        '#open' => TRUE,

        'handler' => [
          '#config_target' => new ConfigTarget($configName, 'imageStyle.listBuilder.handler'),
          '#type' => 'radios',
          '#required' => FALSE,
          '#title' => $this->t('Handler'),
          '#options' => [
            '' => $this->t('- Original -'),
            'devel_visual' => $this->t('Devel Visual'),
          ],
        ],
      ],
    ];

    $form['nodeType'] = [
      '#type' => 'details',
      '#tree' => TRUE,
      '#title' => $this->t('Node Type'),
      '#open' => TRUE,

      'listBuilder' => [
        '#type' => 'details',
        '#tree' => TRUE,
        '#title' => $this->t('List Builder'),
        '#open' => TRUE,

        'handler' => [
          '#config_target' => new ConfigTarget($configName, 'nodeType.listBuilder.handler'),
          '#type' => 'radios',
          '#required' => FALSE,
          '#title' => $this->t('Handler'),
          '#options' => [
            '' => $this->t('- Original -'),
            'devel_visual' => $this->t('Devel Visual'),
          ],
        ],
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * @phpstan-param array<string, mixed> $form
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    parent::submitForm($form, $form_state);

    $configName = $this->getEditableConfigNames()[0];
    $configExportUrl = Url::fromRoute(
      'config.export_single',
      [
        'config_type' => 'system.simple',
        'config_name' => $configName,
      ],
    );

    if ($configExportUrl->access()) {
      $this
        ->messenger()
        ->addStatus($this->t(
          'The saved configuration can be checked <a href=":uri_to_config_export">here</a>.',
          [
            ':uri_to_config_export' => $configExportUrl->toString(),
          ],
        ));
    }
  }

}
