<?php

declare(strict_types=1);

namespace Drupal\devel_visual\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\devel_visual\GraphProviderInterface;
use Fhaculty\Graph\Exporter\ExporterInterface as GraphExporterInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

abstract class PageDependenciesBase extends ControllerBase {

  protected GraphProviderInterface $graphProvider;

  protected GraphExporterInterface $dotExporter;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    // @phpstan-ignore-next-line
    return new static(
      $container->get('devel_visual.graph_exporter.graphviz_dot'),
      $container->get('devel_visual.graph_provider.library_dependencies')
    );
  }

  public function __construct(
    GraphExporterInterface $dotExporter,
    GraphProviderInterface $graphProvider,
  ) {
    $this->dotExporter = $dotExporter;
    $this->graphProvider = $graphProvider;
  }

  /**
   * @phpstan-return array<string, mixed>
   */
  public function content(): array {
    return [
      'graph' => [
        '#markup' => '<div id="my-graph-01"/>',
        '#attached' => [
          'library' => [
            'devel_visual/d3-graphviz.integration',
          ],
          'drupalSettings' => [
            'develVisual' => [
              'd3GraphViz' => [
                'myGraph01' => [
                  'selector' => '#my-graph-01',
                  'source' => [
                    'type' => 'dot',
                    'value' => $this->dotExporter->getOutput($this->graphProvider->getGraph()),
                  ],
                ],
              ],
            ],
          ],
        ],
      ],
    ];
  }

}
