<?php

declare(strict_types=1);

namespace Drupal\devel_visual\CropType;

use Drupal\crop\CropTypeInterface;

class CropTypeAspectRatioComparer {

  public function __invoke(CropTypeInterface $a, CropTypeInterface $b): int {
    return $this->compare($a, $b);
  }

  public function compare(CropTypeInterface $a, CropTypeInterface $b): int {
    return $this->getAspectRatioAsNumber($a) <=> $this->getAspectRatioAsNumber($b);
  }

  public function getAspectRatioAsNumber(CropTypeInterface $cropType): int|float {
    $aspectRatio = $this->getAspectRatio($cropType);

    return $aspectRatio['width'] / $aspectRatio['height'];
  }

  /**
   * @phpstan-return array{width: int, height: int}
   */
  public function getAspectRatio(CropTypeInterface $cropType): array {
    $aspectRatio = $cropType->getAspectRatio();
    if ($aspectRatio) {
      return $this->parseAspectRatio($aspectRatio);
    }

    $aspectRatio = $cropType->getSoftLimit();
    if (!empty($aspectRatio['width']) && !empty($aspectRatio['height'])) {
      return $aspectRatio;
    }

    $aspectRatio = $cropType->getHardLimit();
    if (!empty($aspectRatio['width']) && !empty($aspectRatio['height'])) {
      return $aspectRatio;
    }

    // @todo Weight of crop type without fixed aspect ratio should be configurable.
    // At the beginning of the list or at the end of the list.
    return [
      'width' => 1,
      'height' => 1,
    ];
  }

  /**
   * @phpstan-return array{width: int, height: int}
   */
  public function parseAspectRatio(string $aspectRatio): array {
    $parts = explode(':', $aspectRatio) + [1 => 1];

    return [
      'width' => (int) $parts[0],
      'height' => (int) $parts[1],
    ];
  }

}
