<?php

declare(strict_types=1);

namespace Drupal\devel_visual\FieldType;

class FileListBuilder extends ListBuilderBase {

  protected string $fieldTypeId = 'file';

  public function content() {
    parent::content();

    return $this
      ->contentAddFileRestrictionsColumns()
      ->contentAddFileDirectoryColumns()
      ->build;
  }

  protected function contentAddFileRestrictionsColumns(): static {
    $this->build['content']['#header'] += [
      'file_extensions' => $this->t('Extensions'),
      'max_filesize' => $this->t('Max filesize'),
    ];

    foreach ($this->fieldInstances as $instanceId => $fieldInstance) {
      $this->build['content']['#rows'][$instanceId] += [
        'file_extensions' => [
          'data' => [
            '#markup' => $fieldInstance->getSetting('file_extensions'),
          ],
        ],
        'max_filesize' => [
          'data' => [
            '#markup' => $fieldInstance->getSetting('max_filesize'),
          ],
        ],
      ];
    }

    return $this;
  }

  protected function contentAddFileDirectoryColumns(): static {
    $this->build['content']['#header'] += [
      'uri_scheme' => $this->t('URI scheme'),
    ];

    $fileDirectoryDepth = 0;
    foreach ($this->fieldInstances as $instanceId => $fieldInstance) {
      $fieldId = $fieldInstance->getTargetEntityTypeId() . '.' . $fieldInstance->getName();
      $field = $this->fields[$fieldId];

      $this->build['content']['#rows'][$instanceId]['uri_scheme'] = [
        'data' => [
          '#markup' => $field->getSetting('uri_scheme'),
        ],
      ];

      $fileDirectory = $fieldInstance->getSetting('file_directory');
      $fileDirectoryParts = explode('/', $fileDirectory);
      if ($fileDirectoryDepth < count($fileDirectoryParts)) {
        $fileDirectoryDepth = count($fileDirectoryParts);
      }

      foreach ($fileDirectoryParts as $depth => $fileDir) {
        $this->build['content']['#rows'][$instanceId]["file_directory_$depth"] = [
          'data' => [
            '#markup' => $fileDir,
          ],
        ];
      }
    }

    for ($depth = 0; $depth < $fileDirectoryDepth; $depth++) {
      $this->build['content']['#header']["file_directory_$depth"] = $this->t('Dir @depth', ['@depth' => $depth]);
    }

    foreach ($this->build['content']['#rows'] as &$row) {
      for ($depth = 0; $depth < $fileDirectoryDepth; $depth++) {
        if (isset($row["file_directory_$depth"])) {
          continue;
        }

        $row["file_directory_$depth"] = [
          'data' => [
            '#markup' => '',
          ],
        ];
      }
    }

    return $this;
  }

}
