<?php

declare(strict_types=1);

namespace Drupal\devel_visual\FieldType;

class ImageListBuilder extends FileListBuilder {

  protected string $fieldTypeId = 'image';

  public function content() {
    return $this
      ->initDestination()
      ->initFields()
      ->initBuild()
      ->contentAddBasicColumns()
      ->contentAddFileRestrictionsColumns()
      ->contentAddImageRestrictionsColumns()
      ->contentAddFileDirectoryColumns()
      ->build;
  }

  protected function contentAddImageRestrictionsColumns(): static {
    $this->build['content']['#header'] += [
      'max_resolution' => $this->t('Max resolution'),
      'min_resolution' => $this->t('Min resolution'),
      'alt_field' => $this->t('Enable Alt field'),
      'alt_field_required' => $this->t('Alt field required'),
      'title_field' => $this->t('Enable Title field'),
      'title_field_required' => $this->t('Title field required'),
    ];

    foreach ($this->fieldInstances as $instanceId => $fieldInstance) {
      $this->build['content']['#rows'][$instanceId] += [
        'max_resolution' => [
          'data' => $fieldInstance->getSetting('max_resolution'),
        ],
        'min_resolution' => [
          'data' => $fieldInstance->getSetting('min_resolution'),
        ],
        'alt_field' => [
          'data' => $this->cellDataBoolean($fieldInstance->getSetting('alt_field')),
        ],
        'alt_field_required' => [
          'data' => $this->cellDataBooleanStatus(
            $fieldInstance->getSetting('alt_field'),
            $fieldInstance->getSetting('alt_field_required'),
          ),
        ],
        'title_field' => [
          'data' => $this->cellDataBoolean($fieldInstance->getSetting('title_field')),
        ],
        'title_field_required' => [
          'data' => $this->cellDataBooleanStatus(
            $fieldInstance->getSetting('title_field'),
            $fieldInstance->getSetting('title_field_required'),
          ),
        ],
      ];
    }

    return $this;
  }

}
