<?php

declare(strict_types=1);

namespace Drupal\devel_visual\GraphProvider;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\devel_visual\GraphProviderEvent;
use Drupal\devel_visual\GraphProviderInterface;
use Fhaculty\Graph\Graph;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;

abstract class GraphProviderBase implements GraphProviderInterface, ContainerInjectionInterface {

  /**
   * @abstract
   */
  public const EVENT_ALTER = '';

  protected string $graphName = '';

  protected string $graphRankDir = 'LR';

  protected Graph $graph;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    // @phpstan-ignore-next-line
    return new static(
      $container->get('event_dispatcher')
    );
  }

  public function __construct(
    protected EventDispatcherInterface $eventDispatcher,
  ) {
  }

  public function getGraph(): Graph {
    return $this
      ->initGraph()
      ->buildGraph()
      ->alterGraph()
      ->graph;
  }

  protected function initGraph(): static {
    $this->graph = new Graph();
    $this->graph->setAttribute('graphviz.graph.name', $this->graphName);
    $this->graph->setAttribute('graphviz.graph.rankdir', $this->graphRankDir);

    return $this;
  }

  abstract protected function buildGraph(): static;

  protected function alterGraph(): static {
    $event = new GraphProviderEvent();
    $event->graph = $this->graph;
    $this->eventDispatcher->dispatch($event, static::EVENT_ALTER);

    return $this;
  }

}
